from .coo import Coo


__title__ = "coo"
__license__ = "Apache 2.0"
__version__ = "0.1.2"
__author__ = "Carlos Montecinos Geisse"
__contact__ = "carlos.w.montecinos@gmail.com"
__url__ = "https://github.com/wilfredinni/coo"
